'use client';

import { useEffect, useState } from 'react';
import {
  Connection,
  PublicKey,
  Transaction,
  SystemProgram,
} from '@solana/web3.js';
import axios from 'axios';
import { usePrivy } from '@privy-io/react-auth';
import {
  useSolanaWallets,
} from '@privy-io/react-auth/solana';
import { createClient } from '@supabase/supabase-js';

interface GpuPaymentModalProps {
  isOpen: boolean;
  onClose: () => void;
  gpu: {
    id: string;
    title: string;
    price_usd: number;
    price_per_hour: string; // new field
  };
}


const SOLANA_RPC = process.env.NEXT_PUBLIC_SOLANA_RPC!;
const BUSINESS_WALLET = process.env.NEXT_PUBLIC_BUSINESS_WALLET!;
const EMAIL_API_URL = process.env.NEXT_PUBLIC_EMAIL_API_URL!;

export const GpuPaymentModal = ({ isOpen, onClose, gpu }: GpuPaymentModalProps) => {
  const { user, connectWallet } = usePrivy();
  const { wallets: privyWallets } = useSolanaWallets();

  const [solPrice, setSolPrice] = useState<number | null>(null);
  const [emailInput, setEmailInput] = useState('');
  const [loading, setLoading] = useState(false);
  const [errorMsg, setErrorMsg] = useState<string | null>(null);
  const [successMsg, setSuccessMsg] = useState<string | null>(null);

  const connection = new Connection(SOLANA_RPC);
  const userEmail = user?.email?.address || emailInput || null;

  const isValidEmail = (email: string) => /^[^\s@]+@[^\s@]+\.[^\s@]+$/.test(email);
  const [emailTouched, setEmailTouched] = useState(false);

  const supabase = createClient(
    process.env.NEXT_PUBLIC_SUPABASE_URL!,
    process.env.NEXT_PUBLIC_SUPABASE_ANON_KEY!
  );

  useEffect(() => {
    const fetchSolPrice = async () => {
      try {
        const { data } = await axios.get(
          'https://api.coingecko.com/api/v3/simple/price?ids=solana&vs_currencies=usd'
        );
        setSolPrice(data.solana.usd);
      } catch {
        setErrorMsg('Failed to fetch SOL price.');
      }
    };

    if (isOpen) {
      fetchSolPrice();
      setErrorMsg(null);
      setSuccessMsg(null);
    }
  }, [isOpen]);

  const handlePayment = async () => {
    setLoading(true);
    setErrorMsg(null);
    setSuccessMsg(null);

    try {
      if (!userEmail || !isValidEmail(userEmail)) {
        throw new Error('Please enter a valid email address for your receipt.');
      }

      const solWallet = privyWallets[0];
      
      if (!solWallet || !solWallet.address) {
        throw new Error('No connected Solana wallet found. Please connect a wallet via Privy.');
      }

      if (!solWallet.signTransaction) {
        throw new Error('Connected wallet does not support transaction signing.');
      }

      const fromPubKey = new PublicKey(solWallet.address);
      const toPubKey = new PublicKey(BUSINESS_WALLET);
      
      const solAmount = (gpu.price_usd / (solPrice || 1)).toFixed(6);

      const transaction = new Transaction().add(
        SystemProgram.transfer({
          fromPubkey: fromPubKey,
          toPubkey: toPubKey,
          lamports: Math.floor(parseFloat(solAmount) * 1e9),
        })
      );

      transaction.feePayer = fromPubKey;
      const { blockhash } = await connection.getLatestBlockhash();
      transaction.recentBlockhash = blockhash;

      const signedTx = await solWallet.signTransaction(transaction);

      const txId = await connection.sendRawTransaction(signedTx.serialize());
      await connection.confirmTransaction(txId, 'confirmed');

      // Send confirmation email
      await axios.post(EMAIL_API_URL, {
        email: userEmail,
        product: gpu.title,
        price_hour: gpu.price_per_hour,
        price: gpu.price_usd.toFixed(2),
      }); 

      // Store order in Supabase
      const { error } = await supabase.from('orders').insert([
        {
          gpu_id: gpu.id,
          user_email: userEmail,
          amount_sol: parseFloat(solAmount),
          sol_tx_signature: txId,
          status: 'success',
        },
      ]);

      if (error) {
        throw new Error(`Failed to save order: ${error.message}`);
      }
    

      setSuccessMsg(`Payment successful! Transaction ID: ${txId}`);
    } catch (err: any) {
      setErrorMsg(err.message || 'Payment failed.');
    } finally {
      setLoading(false);
    }
  };

  if (!isOpen) return null;

  const solAmount = solPrice ? (gpu.price_usd / solPrice).toFixed(6) : '...';

  return (
    <div className="fixed inset-0 z-50 flex items-center justify-center bg-black/50">
      <div className="bg-white w-full max-w-md rounded-xl p-6 shadow-xl">
        <h2 className="text-xl font-semibold text-gray-800">{gpu.title}</h2>
        <p className="text-gray-600 mt-1">
          Price: <span className="font-medium">${gpu.price_usd} USD</span> (~{solAmount} SOL)
        </p>

        {errorMsg && (
          <div className="mt-4 rounded-md bg-red-100 px-4 py-2 text-sm text-red-700">
            {errorMsg}
          </div>
        )}

        {successMsg && (
          <div className="mt-4 rounded-md bg-green-100 px-4 py-2 text-sm text-green-700">
            {successMsg}
          </div>
        )}

        {/* Wallet not connected */}
        {privyWallets.length === 0 ? (
          <div className="mt-6 text-center">
            <p className="text-sm text-gray-700 mb-4">
              To proceed with payment, please connect a Solana wallet.
            </p>
            <button
              onClick={connectWallet}
              className="px-4 py-2 rounded-lg bg-indigo-600 text-white hover:bg-indigo-700 transition"
            >
              Connect Wallet
            </button>
          </div>
        ) : (
          <>
            {/* Show email prompt if not logged in with email */}
            {!user?.email?.address && (
              <div className="mt-4">
                <label className="block text-sm text-gray-700 mb-1">
                  Email for invoice & access:
                </label>
                <input
                  type="email"
                  placeholder="you@example.com"
                  value={emailInput}
                  onChange={(e) => setEmailInput(e.target.value)}
                  onBlur={() => setEmailTouched(true)}
                  className="w-full px-3 py-2 border rounded-md text-sm"
                />
                {emailTouched && emailInput && !isValidEmail(emailInput) && (
                  <p className="mt-1 text-xs text-red-600">Please enter a valid email address.</p>
                )}

              </div>
            )}

            <div className="mt-6 flex justify-end gap-3">
              <button
                onClick={onClose}
                disabled={loading}
                className="px-4 py-2 rounded-lg border border-gray-300 text-gray-600 hover:bg-gray-100 transition"
              >
                Cancel
              </button>
              <button
                onClick={handlePayment}
                disabled={
                  loading ||
                  (!user?.email?.address && (!emailInput || !isValidEmail(emailInput)))
                }
                
                className="px-4 py-2 rounded-lg bg-green-600 text-white hover:bg-green-700 transition disabled:opacity-60"
              >
                {loading ? 'Processing...' : 'Confirm Payment'}
              </button>
            </div>
          </>
        )}
      </div>
    </div>
  );
};

export default GpuPaymentModal;
